__all__ = ['IFirmware', 'IFirmware2', 'IUnit', 'UnitType', 'UnitRef']

from typing import Dict, Optional, Tuple, Union


class IDevice:
    pass


class IFirmware:
    @property
    def version(self) -> str:
        raise NotImplementedError

    def write_rw_bit(self, address: int, int1_value: Union[bool, int], ack_tout_sec: float) -> int:
        """
        :returns:
            int1_value

        :raises PsuCtlError:
        """
        raise NotImplementedError

    def write_rw_reg(self, address: int, int16_value: int, ack_tout_sec: float) -> int:
        """
        :returns:
            int16_value

        :raises PsuCtlError:
        """
        raise NotImplementedError


class IFirmware2:
    STAT_CHAN_STEPS = '1'

    CAP_CHAN_MAX_STEPS = '1'
    CAP_CHAN_STEP_TOUT = '2'

    def write(self, query_msg: str) -> int:
        raise NotImplementedError

    def read(self) -> str:
        """
        :returns:
            reply msg
        """
        raise NotImplementedError

    def get_chan_cap(self, chan_ref: str, cap_name: str) -> int:
        raise NotImplementedError

    def get_chan_stat(self, chan_ref: str, stat_name: str) -> int:
        raise NotImplementedError


UnitType = str
UnitRef = Tuple[UnitType, str]


class IUnit:
    def __init__(self, firmware: Optional[IFirmware | IFirmware2], ref: UnitRef):
        self._firmware = firmware
        self._ref = ref

    def run(self, conf: Dict) -> Optional[Tuple[int, int]]:
        raise NotImplementedError
